/***************************************************************************
sp_index_maintenance:	An intelligent Index Maintenance Script 
						that does not use the 
						sys.dm_db_index_physical_stats DMV
						which can have considerable overhead
					
						This script attempts to "questimate"
						the level of fragmentation of an index
						by using the sys.sysindex.rowmodctr and
						dividing it by the total number rows on
						the index. Based on the calculated 
						percentage the stored proc with either
						REORGANIZE if the estimated fragmentation
						is low or REBUILD if the estimated 
						fragmentation is high.
						
						This script has the benefit of not 
						incurring the overhead of 
						sys.dm_db_index_physical_stats and also
						only REBUILD (with MAXDOP=0) or REORGANIZES indexes 
						that have changes since the last time the 
						statistics have been rebuilt.
						
						I would recommend creating a SQL that 
						runs this stored procedure on a weekly
						basis against you Dynamics Database	 
						
						NOTE: Do not forget to set your parameters
						below.  

Witten By:				Michael De Voe 
						Microsoft Certified Master - SQL Server 2008
			
Date:					Jan. 13th, 2011

Version:				1.1

This script is presented "AS IS" and has no warranties expressed or implied!!!
******************************************************************************/
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sp_index_maintenance]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sp_index_maintenance]
GO

CREATE PROCEDURE sp_index_maintenance

AS

DECLARE @RowThreshold INT
DECLARE @PercentThreshold DECIMAL(38,10)

/***********************************************************************************************************************************
*************SET PARAMETERS*********************************************************************************************************/

SET @RowThreshold		= 10000	--Threshold for the number of rows.  In the number of rows in the index is less that the threshold 
								--then it will skip it completely.
SET @PercentThreshold	= .30	--FOR THIS SCRIPT .30 = 30%
								--Threshold for the percentage of rows modified in the index.  If under this number it will RORGANIZE 
								--the index, if it is equal to or greater than this number it REBUILD the Index
						
/************SET PARAMETERS*********************************************************************************************************
************************************************************************************************************************************/

SET NOCOUNT ON

DECLARE @tablename SYSNAME;
DECLARE @index_name SYSNAME;
DECLARE @percentchg DECIMAL(38,10);
DECLARE @command VARCHAR(8000);
DECLARE @command2 VARCHAR(8000);
DECLARE Index_Maint CURSOR FOR

SELECT 
o.name, 
s.name, 
ABS(((CAST(rowmodctr AS DECIMAL(38,10)))/(CAST(rowcnt AS DECIMAL(38,10))))*-1)
FROM sys.sysindexes s 
JOIN sys.objects o ON s.id = o.object_id
WHERE s.rows > 0						--Skip indexes with 0 rows
AND s.rowmodctr > 0						--Skip indexes with 0 row modifications
AND s.name IS NOT NULL					--Skip statistics
AND o.name NOT LIKE ('sys%')			--Skip system indexes
AND s.rowcnt >= @RowThreshold			--Skip indexes with fewer records than the defined threshold

OPEN Index_Maint

FETCH NEXT FROM Index_Maint INTO 
	  @tablename, @index_name, @percentchg

While @@FETCH_STATUS = 0
BEGIN

	IF @percentchg >= @PercentThreshold
		BEGIN
			SELECT @command2 = 'ALTER INDEX ' + '[' + @index_name + ']' + ' ON ' + '[' + @tablename + ']' + ' REBUILD WITH(MAXDOP=0)';
			EXEC (@command2);
			PRINT 'Executed ' + @command2;
		END; 
	ELSE
		BEGIN
			SELECT @command = 'ALTER INDEX ' + '[' + @index_name + ']' + ' ON ' + '[' + @tablename + ']' + ' REORGANIZE';
			EXEC (@command);	
			PRINT 'Executed ' + @command;
		END;			
						
FETCH NEXT FROM Index_Maint INTO 
	  @tablename, @index_name, @percentchg
END 

CLOSE Index_Maint
DEALLOCATE Index_Maint
GO



/************************************************************************************
sp_statistics_maintenance -	This script is essentially just runs 
							sp_updatestats which uses the 
							sys.sysindexes.rowmodctr to determine whether
							a statistics needs to be updated or not.  If 
							data in the statistic has changed then the statistic
							gets updated, and if no data has changed it does not 
							get updated.  This is far more precise and less 
							time consuming than just updating all statistics 
							in a database blindly.
							
							This script just adds code to set MAXDOP to 0 so the
							UPDATE STATITICS operation can make full use of all
							CPUs available to SQL Server, and also because that we
							recommend setting MAXDOP to 1 for all Dynamics
							applications which is best option for OLTP operations
							by not database maintenance operations.  When
							the script is complete it sets MAXDOP back to the
							original value configuration value.
									
							I would recommend creating a SQL that 
							runs this stored procedure on a daily or
							weekly basis against you Dynamics Database
							even if "Auto Update Statistics" is enabled
							(and it better be!!!), because
							"Auto Update Statistics" only executes
							when 5% or more of the data in the statistic
							has changed.  On tables with 10s of millions
							of records this can represent a substantial number
							of rows.  In these situations even a 1% change in
							data can meen the difference between an INDEX SEEK
							and an INDEX plan so these tables must be accounted
							for in any type of database maintenance plan.

								 
				
Witten By:					Michael De Voe 
							Microsoft Certified Master - SQL Server 2008
							
			
Date:						Jan. 13th, 2011

Version:					1.1

This script is presented "AS IS" and has no warranties expressed or implied!!!
**********************************************************************************/
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sp_statistics_maintenance]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sp_statistics_maintenance]
GO

Create Procedure sp_statistics_maintenance

as

DECLARE @MAXDOP INT

CREATE TABLE #Config(
name VARCHAR(250),
minimum INT,
maximum INT,
config_value INT,
run_value INT
)

INSERT #Config
(name,minimum,maximum,config_value,run_value)

EXEC SP_CONFIGURE

SET @MAXDOP = (SELECT config_value FROM #Config WHERE name = 'max degree of parallelism')

EXEC SP_CONFIGURE 'max degree of parallelism', 0
RECONFIGURE WITH OVERRIDE

EXEC SP_UPDATESTATS

EXEC SP_CONFIGURE 'max degree of parallelism', @MAXDOP
RECONFIGURE WITH OVERRIDE

DROP TABLE #Config

GO





















